@@ -6,7 +6,7 @@
 ** of the block to stdout.
 ** 
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -59,12 +59,8 @@
 #define HTML 0x4
 #define STAT 0x8
 
-#define SWAP_TYPE "swap"
-#define PAGESZ	4096
 #define DLS_TYPE "dls"
-#define DLSSZ	512
-#define RAW_TYPE "raw"
-#define RAWSZ 	512
+#define RAW_STR "raw"
 
 void 
 usage(char *myProg)  
@@ -77,13 +73,10 @@ usage(char *myProg)
 	printf("\t-V: display version\n");
 	printf("\t-w: displays in web-like (html) fashion\n");
 	printf("\tIf [bytes] is not given, the default unit size is used\n");
-	printf("\tFor fstype of dls and swap, [bytes] specifies the unit size\n");
 	printf("\t-f fstype: Image file system type\n");
     printf("Supported file system types:\n");
 	fs_print_types();
 	printf("\t%s\n", DLS_TYPE);
-	printf("\t%s\n", SWAP_TYPE);
-	printf("\t%s\n", RAW_TYPE);
 
 	exit(1);
 }
@@ -109,7 +102,6 @@ main(int argc, char **argv)
 	char format = 0;
 	char ch;
 	extern int optind;
-	FILE *hSwap = NULL;
 	progname = argv[0];
 
 	while ((ch = getopt(argc, argv, "af:hsvVw")) > 0) {
@@ -119,6 +111,9 @@ main(int argc, char **argv)
 			break;
 		case 'f':
 			fstype = optarg;
+			if (strcmp(fstype, DLS_TYPE) == 0) 
+				fstype = RAW_STR;
+
 			break;
 		case 'h':
 			format |= HEX;
@@ -147,13 +142,6 @@ main(int argc, char **argv)
 		if (optind + 1 != argc)
 			usage(argv[0]);
 
-		if ((strcmp(SWAP_TYPE, fstype) == 0) ||
-		  (strcmp(DLS_TYPE, fstype) == 0) ||
-		  (strcmp(RAW_TYPE, fstype) == 0) ) {
-			printf("swap/raw/dls type not supported for stat option\n");
-			usage(argv[0]);
-		}
-
 		if (format & (HTML | ASCII | HEX)) {
 			printf ("NOTE: Additional flags will be ignored\n");
 		}
@@ -168,43 +156,29 @@ main(int argc, char **argv)
 		usage(argv[0]);
 	}
 
-	/* open appropriate file */
-	if ((strcmp(SWAP_TYPE, fstype) == 0) ||
-	  (strcmp(RAW_TYPE, fstype) == 0) ||
-	  (strcmp(DLS_TYPE, fstype) == 0)) {
-		if (NULL == (hSwap = fopen(argv[optind++], "r")))
-			error("error opening: %s\n", argv[optind-1]);
-	}
-	else {
-		fs = fs_open(argv[optind++], fstype);
-	}
 
-	/* we already checked above if we had STAT and SWAP, so we don't
-	 * need to check again 
-	 */
+	/* open the file */
+	fs = fs_open(argv[optind++], fstype);
+
 	if (format & STAT) {
 		stats(fs);
 		fs->close(fs);
 		return 0;
 	}
 
+
 	block = atoi(argv[optind++]);
 	if (optind + 1 == argc) {
 		size = atoi(argv[optind++]);
+		if (size < 0) { 
+			error("Invalid size: %i\n", size);
+		} 
+		else if (size % fs->dev_bsize) {
+			error("Size is not a multiple of %d\n", fs->dev_bsize);
+		}
 	}
 	else {
-		if (strcmp(SWAP_TYPE, fstype) == 0) {
-			size = PAGESZ;
-		}
-		else if (strcmp(DLS_TYPE, fstype) == 0) {
-			size = DLSSZ;
-		}
-		else if (strcmp(RAW_TYPE, fstype) == 0) {
-			size = RAWSZ;
-		}
-		else {
-			size = fs->block_size;
-		}
+		size = fs->block_size;
 	}
 
 	if (format & HTML) {
@@ -217,39 +191,17 @@ main(int argc, char **argv)
 
 	}
 
-	/* non swap */
-	if ((strcmp(SWAP_TYPE, fstype) != 0) &&
-	  (strcmp(RAW_TYPE, fstype) != 0) && 
-	  (strcmp(DLS_TYPE, fstype) != 0)) {
-		if (size < 0) { 
-			error("Invalid size: %i\n", size);
-		} 
-		if (size % fs->dev_bsize) {
-			error("Size is not a multiple of %d\n", fs->dev_bsize);
-		}
-	}
-
 	buf = fs_buf_alloc(size);
 
 	/* Read the data */
-	if ((strcmp(SWAP_TYPE, fstype) == 0) ||
-	  (strcmp(RAW_TYPE, fstype) == 0) ||
-	  (strcmp(DLS_TYPE, fstype) == 0)) {
-		if(fseek(hSwap, size*block, SEEK_SET)) 
-			error ("error seeking to: %d\n", size*block);
+	if (block > fs->last_block) {
+		printf("Error: block is larger than last block in image (%lu)\n",
+		  (ULONG)fs->last_block);
+		fs->close(fs);
+		return 1;
+	}
+	fs_read_block(fs, buf, size, block, "");
 
-		if (size != fread(buf->data, 1, size, hSwap)) 
-			error ("error reading file\n");
-	}
-	else {
-		if (block > fs->last_block) {
-			printf("Error: block is larger than last block in image (%lu)\n",
-			  (ULONG)fs->last_block);
-			fs->close(fs);
-			return 1;
-		}
-		fs_read_block(fs, buf, size, block, "");
-	}
 
 	/* do a hexdump like printout */
 	if (format & HEX) {
@@ -345,12 +297,7 @@ main(int argc, char **argv)
 
 	fs_buf_free(buf);
 
-	if ((strcmp(SWAP_TYPE, fstype) == 0) ||
-	  (strcmp(RAW_TYPE, fstype) == 0) ||
-	  (strcmp(DLS_TYPE, fstype) == 0)) 
-		fclose(hSwap);
-	else 
-		fs->close(fs);
+	fs->close(fs);
 
 	if (format & HTML) 
 		printf("</BODY>\n</html>\n");
