@@ -2,7 +2,7 @@
 ** The Sleuth Kit 
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002-2003 Brian Carrier, @stake Inc.  All rights reserved
@@ -192,7 +192,7 @@ ext2fs_dinode_lookup(EXT2FS_INFO *ext2fs, INUM_T inum)
 
     ext2fs->inum = inum;
     if (verbose)
-		printf(
+		fprintf(logfp, 
 		  "%lu m/l/s=%o/%d/%llu u/g=%d/%d macd=%lu/%lu/%lu/%lu\n",
 		  (ULONG) inum, 
 		  getu16(fs, dino->i_mode),
@@ -367,7 +367,7 @@ void
 ext2fs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 			          FS_INODE_WALK_FN action, char *ptr)
 {
-    char   *myname = "ext2fs_inode_walk";
+    char   *myname = "extXfs_inode_walk";
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     GRPNUM_T grpnum;
     UCHAR  *imap = 0;
@@ -453,7 +453,7 @@ void
 ext2fs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
           FS_BLOCK_WALK_FN action, char *ptr)
 {
-    char   *myname = "ext2fs_block_walk";
+    char   *myname = "extXfs_block_walk";
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     FS_BUF *fs_buf = fs_buf_alloc(fs->block_size);
     GRPNUM_T grpnum;
@@ -584,10 +584,7 @@ ext2fs_file_walk_direct(FS_INFO *fs, FS_BUF *buf[],
     int     read_count;
 	int 	myflags;
 
-	if (flags & FS_FLAG_SLACK)
-		read_count = buf[0]->size;
-	else
-    	read_count = (length < buf[0]->size ? length : buf[0]->size);
+    read_count = (length < buf[0]->size ? length : buf[0]->size);
 
 	if (addr > fs->last_block) {
 		if (flags & FS_FLAG_NOABORT)  {
@@ -638,7 +635,7 @@ static int
 ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
   DADDR_T addr, int level, int flags, FS_FILE_WALK_FN action, char *ptr)
 {    
-    char   *myname = "ext2fs_file_walk_indir";
+    char   *myname = "extXfs_file_walk_indir";
     size_t   todo_count = length;
     DADDR_T *iaddr;
     int     n;
@@ -739,10 +736,16 @@ ext2fs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
     buf = (FS_BUF **) mymalloc(sizeof(*buf) * (inode->indir_count + 1));
     buf[0] = fs_buf_alloc(fs->file_bsize);
 
+    length = inode->size;
+
+	/* Roundup if we want the slack space on the final fragment */
+	if (flags & FS_FLAG_SLACK)
+		length = roundup(length, fs->block_size);
+
     /*
      * Read the file blocks. First the direct blocks, then the indirect ones.
      */
-    length = inode->size;
+
     for (n = 0; length > 0 && n < inode->direct_count; n++) {
     	retval = ext2fs_file_walk_direct(fs, buf, length, 
 		  inode->direct_addr[n],  flags, action, ptr);
@@ -1212,6 +1215,8 @@ ext2fs_close(FS_INFO *fs)
 	free((char *)ext2fs->fs);
 	free((char *)ext2fs->dinode);
 	free((char *)ext2fs->group);
+	free((char *)ext2fs->block_map);
+	free((char *)ext2fs->inode_map);
     free(ext2fs);
 }
 
@@ -1220,7 +1225,7 @@ ext2fs_close(FS_INFO *fs)
 FS_INFO *
 ext2fs_open(const char *name, unsigned char ftype)
 {
-    char   *myname = "ext2fs_open";
+    char   *myname = "extXfs_open";
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) mymalloc(sizeof(*ext2fs));
     int     len;
 	FS_INFO *fs = &(ext2fs->fs_info);
