@@ -1039,46 +1039,132 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 	u_int32_t	next, snext, sstart, send;
 	FATFS_INFO *fatfs = (FATFS_INFO *)fs;
 	fatfs_sb *sb = fatfs->sb;
+	FS_BUF *fs_buf = fs_buf_alloc(fatfs->ssize);
+	fatfs_dentry *de;
+
+	/* Read the root directory sector so that we can get the volume
+	 * label from it */
+	fs_read_block(fs, fs_buf, fatfs->ssize, 
+	  fatfs->rootsect, "root directory");
+
+	/* Find the dentry that is set as the volume label */
+	de = (fatfs_dentry *)fs_buf->data;
+	for (i = 0; i < fatfs->ssize; i += sizeof (*de)) {
+		if (de->attrib == FATFS_ATTR_VOLUME) 
+			break;
+		de++;
+	}
+	/* If we didn't find it, then reset de */
+	if (de->attrib != FATFS_ATTR_VOLUME) 
+		de = NULL;
+
+
+
+	/* PRint the general file system information */
 
 	fprintf(hFile, "FILE SYSTEM INFORMATION\n");
 	fprintf(hFile, "--------------------------------------------\n");
 
 	fprintf(hFile, "File System Type: FAT\n");
 
-	fprintf(hFile, "OEM: %c%c%c%c%c%c%c%c\n", sb->oemname[0], 
+	fprintf(hFile, "\nOEM Name: %c%c%c%c%c%c%c%c\n", sb->oemname[0], 
 	  sb->oemname[1], sb->oemname[2], sb->oemname[3], sb->oemname[4],
 	  sb->oemname[5], sb->oemname[6], sb->oemname[7]);
 	
 
 	if (fatfs->fs_info.ftype != MS32_FAT) {
-		fprintf(hFile, "Volume ID: %lu\n", (ULONG)getu32(fs, sb->a.f16.vol_id));
-		fprintf(hFile, "Volume Label: %c%c%c%c%c%c%c%c%c%c%c\n", 
+		fprintf(hFile, "Volume ID: 0x%x\n", getu32(fs, sb->a.f16.vol_id));
+
+		fprintf(hFile, "Volume Label (Super Block): %c%c%c%c%c%c%c%c%c%c%c\n", 
 		  sb->a.f16.vol_lab[0], sb->a.f16.vol_lab[1], sb->a.f16.vol_lab[2],
 		  sb->a.f16.vol_lab[3], sb->a.f16.vol_lab[4], sb->a.f16.vol_lab[5],
 		  sb->a.f16.vol_lab[6], sb->a.f16.vol_lab[7], sb->a.f16.vol_lab[8],
 		  sb->a.f16.vol_lab[9], sb->a.f16.vol_lab[10]);
 
-		fprintf(hFile, "File System Type (super block): %c%c%c%c%c%c%c%c\n", 
+		if ((de) && (de->name)) {
+			fprintf(hFile, "Volume Label (Root Directory): %c%c%c%c%c%c%c%c%c%c%c\n", 
+			  de->name[0], de->name[1], de->name[2], de->name[3],
+			  de->name[4], de->name[5], de->name[6], de->name[7],
+			  de->name[8], de->name[9], de->name[10]);
+		} else {
+			fprintf(hFile, "Volume Label (Root Directory):\n");
+		}
+
+		fprintf(hFile, "File System Type Label: %c%c%c%c%c%c%c%c\n", 
 		  sb->a.f16.fs_type[0], sb->a.f16.fs_type[1], sb->a.f16.fs_type[2],
 		  sb->a.f16.fs_type[3], sb->a.f16.fs_type[4], sb->a.f16.fs_type[5],
 		  sb->a.f16.fs_type[6], sb->a.f16.fs_type[7]);
 	}
 	else {
 
-		fprintf(hFile, "Volume ID: %lu\n", (ULONG)getu32(fs, sb->a.f32.vol_id));
-		fprintf(hFile, "Volume Label: %c%c%c%c%c%c%c%c%c%c%c\n", 
+    	FS_BUF *fat_fsinfo_buf = fs_buf_alloc(sizeof(fatfs_fsinfo));
+		fatfs_fsinfo *fat_info;
+
+		fprintf(hFile, "Volume ID: 0x%x\n", getu32(fs, sb->a.f32.vol_id));
+
+		fprintf(hFile, "Volume Label (Super Block): %c%c%c%c%c%c%c%c%c%c%c\n", 
 		  sb->a.f32.vol_lab[0], sb->a.f32.vol_lab[1], sb->a.f32.vol_lab[2],
 		  sb->a.f32.vol_lab[3], sb->a.f32.vol_lab[4], sb->a.f32.vol_lab[5],
 		  sb->a.f32.vol_lab[6], sb->a.f32.vol_lab[7], sb->a.f32.vol_lab[8],
 		  sb->a.f32.vol_lab[9], sb->a.f32.vol_lab[10]);
 
-		fprintf(hFile, "File System Type: %c%c%c%c%c%c%c%c\n", 
+		if ((de) && (de->name)) {
+			fprintf(hFile, "Volume Label (Root Directory): %c%c%c%c%c%c%c%c%c%c%c\n", 
+			  de->name[0], de->name[1], de->name[2], de->name[3],
+			  de->name[4], de->name[5], de->name[6], de->name[7],
+			  de->name[8], de->name[9], de->name[10]);
+		} else {
+			fprintf(hFile, "Volume Label (Root Directory):\n");
+		}
+
+		fprintf(hFile, "File System Type Label: %c%c%c%c%c%c%c%c\n", 
 		  sb->a.f32.fs_type[0], sb->a.f32.fs_type[1], sb->a.f32.fs_type[2],
 		  sb->a.f32.fs_type[3], sb->a.f32.fs_type[4], sb->a.f32.fs_type[5],
 		  sb->a.f32.fs_type[6], sb->a.f32.fs_type[7]);
 
+		fprintf(hFile, "\nBackup Boot Sector Location: %lu\n",
+		  (ULONG)getu32(fs, sb->a.f32.bs_backup));
+
+		fprintf(hFile, "FS Info Sector Location: %lu\n",
+		  (ULONG)getu16(fs, sb->a.f32.fsinfo));
+
+		fs_read_block(fs, fat_fsinfo_buf, sizeof(fatfs_fsinfo), 
+		  getu16(fs, sb->a.f32.fsinfo), 
+		  "read fs_info");
+
+		fat_info = (fatfs_fsinfo *)fat_fsinfo_buf->data;
+		fprintf(hFile, "Next Free Sector: %lu\n",
+		  (ULONG)
+		  (fatfs->firstclustsect + 
+		  (getu32(fs, fat_info->nextfree) - 2) * fatfs->csize));
+
+		fprintf(hFile, "Free Sector Count: %lu\n",
+		  (ULONG)
+		  (getu32(fs, fat_info->freecnt) * fatfs->csize));
+
+		fs_buf_free(fat_fsinfo_buf);
 	}
 
+	fs_buf_free(fs_buf);
+
+	fprintf(hFile, "\nSectors before file system: %d\n",
+		getu32(fs, sb->prevsect));
+
+	fprintf(hFile, "Reserved Sector Range: 0 - %d\n",
+	  fatfs->firstfatsect - 1);
+
+	for (i = 0; i < fatfs->numfat; i++) {
+		u_int32_t base = fatfs->firstfatsect + i * (fatfs->sectperfat);
+
+		fprintf(hFile, "FAT %d Sector Range: %lu - %lu\n", i, (ULONG)base,
+		   (ULONG)(base + fatfs->sectperfat - 1));
+	}
+
+	fprintf(hFile, "Data Area Sector Range: %lu - %lu\n", 
+	  (ULONG)fatfs->firstdatasect, (ULONG)fs->last_block);
+
+
+
 	fprintf(hFile, "\nMETA-DATA INFORMATION\n");
 	fprintf(hFile, "--------------------------------------------\n");
 
@@ -1096,15 +1182,6 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 	fprintf(hFile, "Total Sector Range: %lu - %lu\n", (ULONG)fs->start_block, 
 	  (ULONG)fs->last_block);
 
-	for (i = 0; i < fatfs->numfat; i++) {
-		u_int32_t base = fatfs->firstfatsect + i * (fatfs->sectperfat);
-
-		fprintf(hFile, "FAT %d Range: %lu - %lu\n", i, (ULONG)base,
-		   (ULONG)(base + fatfs->sectperfat - 1));
-	}
-
-	fprintf(hFile, "Data Area Sector Range: %lu - %lu\n", 
-	  (ULONG)fatfs->firstdatasect, (ULONG)fs->last_block);
 
 	/* Display the FAT Table */
 	fprintf(hFile, "\nFAT CONTENTS (in sectors)\n");
@@ -1253,33 +1330,33 @@ fatfs_close(FS_INFO *fs)
 FS_INFO *
 fatfs_open(const char *name, unsigned char ftype)
 {
-    char   		*myname = "fatfs_open";
-    FATFS_INFO 	*fatfs = (FATFS_INFO *) mymalloc(sizeof(*fatfs));
-    int     	len;
-    FS_INFO 	*fs = &(fatfs->fs_info);
+	char   		*myname = "fatfs_open";
+	FATFS_INFO 	*fatfs = (FATFS_INFO *) mymalloc(sizeof(*fatfs));
+	int     	len;
+	FS_INFO 	*fs = &(fatfs->fs_info);
 	fatfs_sb	*fatsb;
 
 	
-    if ((ftype & FSMASK) != FATFS_TYPE)
-        error ("%s: Invalid FS Type in fatfs_open", myname);
+	if ((ftype & FSMASK) != FATFS_TYPE)
+		error ("%s: Invalid FS Type in fatfs_open", myname);
 
-    if ((fs->fd = open(name, O_RDONLY)) < 0)
-        error("%s: open %s: %m", myname, name);
+	if ((fs->fd = open(name, O_RDONLY)) < 0)
+		error("%s: open %s: %m", myname, name);
 
-    fs->ftype = ftype;
-    
-    /*
-     * Read the super block.
-     */
-    len = sizeof(fatfs_sb);
-    fatsb = fatfs->sb = (fatfs_sb *)mymalloc (len);
+	fs->ftype = ftype;
 
-    if (read(fs->fd, (char *)fatsb, len) != len)
-        error("%s: read superblock: %m", name);
+	/*
+	* Read the super block.
+	*/
+	len = sizeof(fatfs_sb);
+	fatsb = fatfs->sb = (fatfs_sb *)mymalloc (len);
+
+	if (read(fs->fd, (char *)fatsb, len) != len)
+		error("%s: read superblock: %m", name);
 
 	/* Check the magic value  and ID endian ordering */
 	if (guessu16(fs, fatsb->magic, FATFS_FS_MAGIC)) 
-			error ("Error: %s is not a FATFS file system", name);
+		error ("Error: %s is not a FATFS file system", name);
 
 	fatfs->ssize = getu16(fs, fatsb->ssize);
 	if (fatfs->ssize % FATFS_DEV_BSIZE) {
@@ -1303,10 +1380,6 @@ fatfs_open(const char *name, unsigned char ftype)
 	if (0 == (fatfs->sectors = getu16(fs, fatsb->sectors16)))
 		fatfs->sectors = getu32(fs, fatsb->sectors32);
 
-
-	/* for some reason, on my test images this number was always 1 too large */
-	fatfs->sectors--;
-
 	/* if secperfat16 is 0, then read sectperfat32 */
 	if (0 == (fatfs->sectperfat = getu16(fs, fatsb->sectperfat16)))
 		fatfs->sectperfat = getu32(fs, fatsb->a.f32.sectperfat32);
