@@ -81,7 +81,8 @@ typedef struct {
 	u_int8_t	sectors16[2];/* number of sectors in FS */
 	u_int8_t	f2[1];
 	u_int8_t	sectperfat16[2]; /* size of FAT */
-	u_int8_t	f3[8];
+	u_int8_t	f3[4];
+	u_int8_t	prevsect[4];	/* number of sectors before FS partition */
 	u_int8_t	sectors32[4];	/* 32-bit value of number of FS sectors */
 
 	/* The following are different for fat12/fat16 and fat32 */
@@ -98,7 +99,9 @@ typedef struct {
 			u_int8_t	ext_flag[2];
 			u_int8_t	fs_ver[2];
 			u_int8_t	rootclust[4];	/* cluster where root directory is stored */
-			u_int8_t	f5[16];
+			u_int8_t	fsinfo[2];	/* FS_INFO Location */
+			u_int8_t	bs_backup[2];  /* sector of backup of boot sector */
+			u_int8_t	f5[12];
 			u_int8_t	drvnum;
 			u_int8_t	f6[2];
 			u_int8_t	vol_id[4];
@@ -112,6 +115,17 @@ typedef struct {
 
 } fatfs_sb;
 
+typedef struct {
+	u_int8_t	magic1[4];	/* 41615252 */
+	u_int8_t	f1[480];
+	u_int8_t	magic2[4];	/* 61417272 */
+	u_int8_t	freecnt[4];		/* free clusters 0xfffffffff if unknown */
+	u_int8_t	nextfree[4];	/* next free cluster */
+	u_int8_t	f2[12];
+	u_int8_t	magic3[4];  /* AA550000 */
+} fatfs_fsinfo;
+
+
 
 /* directory entry short name structure */
 typedef struct {
