@@ -2,7 +2,7 @@
 ** The Sleuth Kit 
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002-2003 Brian Carrier, @stake Inc.  All rights reserved
@@ -610,11 +610,7 @@ ffs_file_walk_direct(FS_INFO *fs, FS_BUF *buf,
     int     read_count;
     int     myflags;
 
-	/* Get the whole block if we want slack */
-    if (flags & FS_FLAG_SLACK)
-        read_count = buf->size;
-    else
-        read_count = (length < buf->size ? length : buf->size);
+	read_count = (length < buf->size ? length : buf->size);
 
     if (addr > fs->last_block) {
         if (flags & FS_FLAG_NOABORT)  {
@@ -779,10 +775,14 @@ ffs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
     buf = (FS_BUF **) mymalloc(sizeof(*buf) * (inode->indir_count + 1));
     buf[0] = fs_buf_alloc(fs->file_bsize);
     
+    length = inode->size;
+	/* If we want the slack of the last fragment, then roundup */
+    if (flags & FS_FLAG_SLACK)
+		length = roundup(length, fs->block_size);
+
     /*
      * Read the file blocks. First the direct blocks, then the indirect ones.
      */     
-    length = inode->size;
     for (n = 0; length > 0 && n < inode->direct_count; n++) {
         retval = ffs_file_walk_direct(fs, buf[0], length, 
 		  inode->direct_addr[n], flags, action, ptr);
