@@ -182,6 +182,21 @@ extern int optind;
 
 #endif /* CYGWIN */
 
+#if defined(__INTERNIX)
+#define SUPPORTED
+#define LSEEK		lseek
+#define OFF_T		off_t
+#define STRTOUL		strtoul
+#define DADDR_T		long 
+#define DEF_FSTYPE	"freebsd"
+#define INO_TO_CG	ino_to_cg
+
+#define roundup(x, y)	\
+	( ( ((x)+((y) - 1)) / (y)) * (y) )
+
+#endif /* INTERNIX*/
+
+
 
 
 
@@ -528,6 +543,16 @@ extern FS_INFO *fatfs_open(const char *, unsigned char ftype);
  */
 extern FS_INFO *ntfs_open(const char *, unsigned char ftype);
 
+/*
+ * Support for RAW
+ */
+extern FS_INFO *rawfs_open(const char *, unsigned char ftype);
+
+/*
+ * Support for SWAP
+ */
+extern FS_INFO *swapfs_open(const char *, unsigned char ftype);
+
 
  /*
   * Support for long seeks.
