@@ -52,4 +52,10 @@ extern void fs_print_types();
 #define NTFS_TYPE		0x40
 #define NTFS         0x40
 
+#define	SWAPFS_TYPE		0x50
+#define	SWAP			0x50
+
+#define	RAWFS_TYPE		0x60
+#define	RAW				0x60
+
 #endif
